#!/usr/bin/env python

import visvis as vv
app = vv.use()

f = vv.clf()
a = vv.cla()


vv.plot([12,34,21,38], lc='b', ls=':',mc='b', mw=7, lw=2, ms='s', mec='r')
vv.plot([1,3,4],[33,47,12], lc='r', mc='r', ms='.')
vv.plot([20,24,45,21], lc='g', ls='--', mc='g', mw=12, lw=3, ms='')
vv.plot([35,14,40,31], lc='k', ls='-.', mc='g', mw=12, lw=3, ms='*')
# If the star is not visible, your OpenGl system does not support point sprites

a = vv.gca()
a.legend = 'line 1', 'line 2', 'line 3'
a.axis.showGrid = 1

a.axis.xlabel = 'measurement number'
a.axis.ylabel = 'some quantity [unit]'
vv.title('An example of \b{plotting}')

app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Package visvis.backends

Visvis allows multiple backends. I tried to make implementing a
backend as easy as possible. Each backend is defined in a single
module called "backend_xx.py", where xx is the backend name, which
we recommend making all lowercase.

This module should ...
1.  ... contain a class called "Figure" (inherited from visvis.BaseFigure).
    The Figure class should wrap (not inherit from) a widget native to
    the backend, in which visvis can draw with OpenGl. The Figure class
    should also overload a number of functions (such as _SwapBuffers()).
2.  ... contain a function called "newFigure". This function should generate
    a window with a single Figure widget in it, and return the Figure
    Object.
3.  Contain a class called "App" (inherited from vv.events.App) that
    implements the methods "_GetNativeApp", "_ProcessEvents" and "_Run".
    An instance of this class called "app" should be in the module namespace.
4.  ... call vv.events.processVisvisEvents on a regular basis (every 10 ms
    or so). To keep visvis' own event system running.
5.  ... pass through the events enter, leaver, keydown, keyup, resize, close
    via visvis' event system. Pass through events mouseDown, mouseUp,
    mouseDoubleClick, mouseMove via the figure's _GenerateMouseEvent() method,
    that will fire the events of the appropriate wibjects and wobjects.

There is a wiki page about implementing backend:
https://github.com/almarklein/visvis/wiki/Creating_a_backend
Also look at the already implemented backends!

The backend is chosen/selected as follows:
- A user can call vv.use() to load a specific backend and obtain the
  App instance.
- When a figure is created it is checked whether a backend is already
  selected. If not, one is selected automatically; it tries loading the
  backends in the order that is defined in the variable "backendOrder"
  in this file.

"""

# An overview:
# - testLoaded tests to see whether any of the backends is already loaded
# - _loadBackend imports the backend and sets newFigure and app
# - use is the user friendly function that calls _loadBackend

import os
import sys
import imp
import visvis
from visvis.core.misc import isFrozen, getExceptionInstance

# The order in which to try loading a backend (foo is a dummy backend)
backendOrder = ['pyqt5', 'pyside2', 'pyside', 'pyqt4', 'wx', 'gtk', 'fltk']
backendMap = {'pyqt5':'PyQt5',
              'pyside':'PySide',
              'pyside2':'PySide2',
              'pyqt4':'PyQt4',
              'wx':'wx',
              'gtk':'gtk',
              'fltk':'fltk',
              }

# Define aliases for backend names (for backward compatibility)
backendAliases = {'qt4': 'pyqt4', 'qt5':'pyqt5'}

# Set Qt lib to empty string (will be set by backend_pyside or backend_pyqt4)
qtlib = ''

# Put the preferredBackend in front
if True:
    be = visvis.settings.preferredBackend
    be = backendAliases.get(be, be)
    if be in backendOrder:
        backendOrder.remove(be)
        backendOrder.insert(0,be)

# Establish preference based on loaded backends modules
# In this way, in an interactive interpreter the right backend is picked
if visvis.settings.preferAlreadyLoadedBackend:
    for be in [be for be in reversed(backendOrder)]:
        # Determine backend module name
        modName = backendMap[be]
        # If loaded, move up front
        if modName in sys.modules:
            backendOrder.remove(be)
            backendOrder.insert(0,be)


# Placeholder
class BackendDescription:
    def __init__(self):
        self.name = ''
        self.newFigure = None
        self.app = None
currentBackend = BackendDescription()

# Set __file__ absolute when loading
__file__ = os.path.abspath(__file__)

def testLoaded():
    """ Tests to see whether a backend is already loaded.
    Returns the name of the loaded backend, or an empty
    string if nothing is loaded.
    If visvis is part of a frozen app, returns "" always.
    """
    
    if isFrozen():
        return ""
    
    else:
        
        # see which backends we have
        path = __file__
        path = os.path.dirname( path )
        files = os.listdir(path)
        
        for filename in files:
            if not filename.startswith('backend_'):
                continue
            if not filename.endswith('.py'):
                continue
            be = filename[:-3]
            modNameFull = 'visvis.backends.' + be
            if modNameFull in sys.modules:
                i = be.find('_')
                return be[i+1:]
        # nothing found...
        return ''


def _loadBackend(name):
    """ Load the backend with the specified name.
    Returns True on success and False otherwise.
    """
    
    # Get module names
    modName = 'backend_' + name
    modNameFull = 'visvis.backends.backend_'+name
    modFileName = os.path.join(os.path.dirname(__file__), modName+'.py')
    
    # Test whether to use the pyc version
    if not os.path.isfile(modFileName):
        modFileName += "c" # Probably a frozen app
    
    # Try importing the backend toolkit
    try:
        modNameTk = backendMap[name]
        __import__(modNameTk)
    except ImportError:
        return False
    
    # Try importing the visvis backend module
    try:
        if modFileName.endswith('.pyc'):
            module = __import__(modNameFull, fromlist=[modName])
        else:
            module = imp.load_source(modNameFull, modFileName)
        globals()[modName] = module
    except Exception:
        if not isFrozen():
            # Only show if not frozen. If frozen, it can easily happen that
            # the GUI toolkit is available, but the visvis backend is not.
            why = str(getExceptionInstance())
            print('Error importing %s backend: %s' % (name, why))
        return False
    
    # Do some tests
    if not hasattr(module,'newFigure'):
        raise Exception("Backend %s does not implement newFigure!" % be)
    elif not hasattr(module,'Figure'):
        raise Exception("Backend %s does not implement Figure class!" % be)
    else:
        # All good (as far as I can tell). Update stuff if name does not match.
        module.app._backend = name
        if currentBackend.name != name:
            currentBackend.name = name
            currentBackend.newFigure = module.newFigure
            currentBackend.app = module.app
        return True


def use(backendName=None):
    """ use(backendName=None)
    
    Use the specified backend and return an App instance that has a run()
    method to enter the GUI toolkit's mainloop.
    
    If no backend is given, a suitable backend is tried automatically.
    
    """
    
    # Make case insensitive and check
    if backendName:
        backendName = backendName.lower()
        backendName = backendAliases.get(backendName, backendName)
        if backendName not in backendOrder:
            raise RuntimeError('Invalid backend name given: "%s".'%backendName)
    
    # Get name of the backend currently loaded (can be '')
    loadedName = testLoaded()
    
    # Prevent resetting the backend to use
    if loadedName and backendName and loadedName != backendName:
        print("Warning: changing backend while %s already loaded." % loadedName)
        #raise RuntimeError("Cannot change backend, %s already loaded." % loadedName)
    
    # Process
    if backendName:
        # Use given
        if not _loadBackend(backendName):
            tmp = backendName
            tmp = os.path.join(os.path.dirname(__file__), backendName+'.xxx')
            raise RuntimeError('Given backend "%s" could not be loaded.' % tmp)
    elif loadedName:
        # Use loaded (redo to make sure the placeholder is ok)
        if not _loadBackend(loadedName):
            raise RuntimeError('Could not reload backend "%s".' % loadedName)
    else:
        # Try any backend
        for name in backendOrder:
            if _loadBackend(name):
                break
        else:
            tmp = "Install PySide2, PyQt5, PySide, PyQt4, wxPython, GTK, or fltk."
            raise RuntimeError("None of the backends could be loaded. "+tmp)
    
    # Return instance
    return currentBackend.app



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np

def hist(data, bins=None, drange=None, normed=False, weights=None):
    """ hist(a, bins=None, range=None, normed=False, weights=None)
    
    Make a histogram plot of the data. Uses np.histogram (new version)
    internally. See its docs for more information.
    
    See the kde() function for a more accurate density estimate.
    See the vv.StatData for more statistics on data.
    
    Parameters
    ----------
    a : array_like
        The data to calculate the historgam of.
    bins : int or sequence of scalars, optional
        If `bins` is an int, it defines the number of equal-width bins in
        the given range. If `bins` is a sequence, it defines the bin edges,
        including the rightmost edge, allowing for non-uniform bin widths.
        If bins is not given, the best number of bins is determined
        automatically using the Freedman-Diaconis rule.
    range : (float, float)
        The lower and upper range of the bins. If not provided, range is
        simply (a.min(), a.max()). Values outside the range are ignored.
    normed : bool
        If False, the result will contain the number of samples in each bin.
        If True, the result is the value of the probability *density*
        function at the bin, normalized such that the *integral* over the
        range is 1. Note that the sum of the histogram values will not be
        equal to 1 unless bins of unity width are chosen; it is not a
        probability *mass* function.
    weights : array_like
        An array of weights, of the same shape as `a`. Each value in `a`
        only contributes its associated weight towards the bin count
        (instead of 1). If `normed` is True, the weights are normalized,
        so that the integral of the density over the range remains 1.
    
    """
    
    # Auto determine bins?
    if bins is None:
        from visvis.processing.statistics import StatData
        stats = StatData(data)
        bins = stats.best_number_of_bins()
    
    # let numpy do the work
    values, edges = np.histogram(data, bins, drange, normed, weights)
    
    # the bins are the left bin edges, let's get the centers
    centers = np.empty(values.shape, np.float64)
    for i in range(len(values)):
        centers[i] = (edges[i] + edges[i+1]) * 0.5
    
    # plot
    dbin = centers[1] - centers[0]
    return vv.bar(centers, values, width=dbin*0.9)
    #return vv.plot(centers, values, **kwargs)


if __name__ == '__main__':
    vv.clf()
    data = np.random.normal(7,2,size=(100,100))
    b = vv.hist(data)
    b.color = 'r'



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv


class Recorder:
    """ Recorder(object)
    
    Recorder class that makes snapshots right after each draw event. Object
    should be an Axes, AxesContainer or Figure.
    
    It is then possible to export the movie to SWF, GIF, AVI, or a series
    of images.
    
    See also vv.movieWrite().
    
    """
    
    def __init__(self, ob):
        # init
        self._ob = ob
        self._frames = []
        
        # register events
        f = ob.GetFigure()
        f.eventAfterDraw.Bind(self._OnAfterDraw)
    
    
    def _OnAfterDraw(self, event):
        im = vv.getframe(self._ob)
        self._frames.append(im)
    
    
    def Clear(self):
        """ Clear()
        Clear all recorded images up to now.
        """
        self._frames[:] = []
    
    
    def Stop(self):
        """ Stop()
        Stop recording. """
        f = self._ob.GetFigure()
        f.eventAfterDraw.Unbind(self._OnAfterDraw)
    
    
    def Continue(self):
        """ Continue()
        Continue recording. """
        f = self._ob.GetFigure()
        f.eventAfterDraw.Unbind(self._OnAfterDraw)
        f.eventAfterDraw.Bind(self._OnAfterDraw)
    
    
    def GetFrames(self):
        """ GetFrames()
        Get a copy of the list (the frames itself are not copied)
        recorded up to now. """
        return [frame for frame in self._frames]
    
    
    def Export(self, filename, duration=0.1, repeat=True, **kwargs):
        """ Export(self, filename, duration=0.1, repeat=True, **kwargs)
        
        Export recorded movie to either:
          * a series of images
          * an animated GIF
          * an SWF (shockwave flash) file
          * an AVI file
        
        See vv.movieWrite for more information.
        
        """
        frames = self.GetFrames()
        if filename.lower().endswith('.gif'):
            loop = 0 if repeat else 1
            vv.movieWrite(filename, frames, fps=1/duration, loop=loop, **kwargs)
        else:
            vv.movieWrite(filename, frames, fps=1/duration, **kwargs)


def record(ob):
    """ record(object)
    Take a snapshot of the given figure or axes after each draw.
    A Recorder instance is returned, with which the recording can
    be stopped, continued, and exported to GIF, SWF or AVI.
    """
    
    # establish wheter we can record that
    if not isinstance(ob, (vv.BaseFigure, vv.Axes)):
        raise ValueError("The given object is not a figure nor an axes.")
    
    # create recorder
    return Recorder(ob)


if __name__ == '__main__':
    import time
    l = vv.plot([1,2,3,1,4])
    rec = vv.record(vv.gcf())
    for i in range(20):
        l.SetYdata([1+i/10.0, 2,3,1,4])
        vv.processEvents() # Process gui events
        time.sleep(0.1)
    # Export to swf, gif or avi
    fname = 'recordExample' # note: set location to an existing directory
    for ext in ['.swf', '.gif', '.avi']:
        try:
            rec.Export(fname+ext)
        except Exception:
            print('could not do %s' % ext)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import visvis.backends as backends
from visvis import BaseFigure


def figure(fig=None, clear=False):
    """ figure(fig=None, clear=False)

    Set the specified figure to be the current figure, creating it if
    necessary.  fig may be a Figure object, a figure number (a positive
    integer), or None.  Returns the specified or created figure.

    """

    # check if backends are loaded
    if not backends.currentBackend.name:
        backends.use()

    # get function to create new figure
    newFigure = backends.currentBackend.newFigure

    # fig can be a Figure instance
    if isinstance(fig, BaseFigure):
        if fig._destroyed:
            raise Exception("Figure has already been destroyed.")
        nr = fig.nr
    # ... or a positive integer
    elif fig is not None:
        # test nr
        try:
            nr = int(fig)
            if nr <= 0:
                raise ValueError()
        except (ValueError, TypeError):
            raise Exception("Figure number should be an integer >=1")
    else:
        nr = None

    # does a figure with that number already exist?
    if nr and nr in BaseFigure._figures:
        # make current return that
        fig = BaseFigure._figures[nr]
        BaseFigure._currentNr = nr
        if clear:
            fig.Clear()
        return fig
    else:
        if nr:
            # prepare spot, if no nr given, a spot is chosen in the
            # constructor of BaseFigure...
            BaseFigure._figures[nr] = None
        # create new figure and return
        fig = newFigure()
        fig.title = '' #_SetTitle("Figure " + str(fig.nr))
        return fig


if __name__ == '__main__':
    fig = vv.figure()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Package visvis.functions

Each module in this directory contains a function with the same
name as the module it's in. These functions are automatically loaded
in Visvis, thus making it very easy to expand Visvis' functionality.

There are three things to take into account when making a new function:
- The function to add should be the same name as the module in which
  it is defined. Other helper functions or classes can be defined, but
  these are not inserted in the Visvis namespace.
- The function's docstring should be very descriptive and starting with
  the function's signature. The docstring is also used to produce the
  on-line documentation.


"""

import os
import zipfile

from visvis.core.misc import splitPathInZip


def _insertFunctions():
    """ insert the function is this module's namespace.
    """
    
    # see which files we have
    path = __file__
    path = os.path.dirname( os.path.abspath(path) )
    
    # determine if we're in a zipfile
    zipfilename, path = splitPathInZip(path)
    
    if zipfilename:
        # get list of files from zipfile
        z = zipfile.ZipFile(zipfilename)
        files = [os.path.split(i)[1] for i in z.namelist()
                    if i.startswith('visvis') and i.count('functions')]
    else:
        # get list of files from file system
        files = os.listdir(path)
    
    # extract all functions
    names = []
    for file in files:
        # not this file
        if file.startswith('__'):
            continue
        # only python files
        if file.endswith('.pyc'):
            if file[:-1] in files:
                continue # only try import once
        elif not file.endswith('.py'):
            continue
        # build name
        funname = os.path.splitext(file)[0]
        # import module
        mod = __import__("visvis.functions."+funname, fromlist=[funname])
        if not hasattr(mod,funname):
            print("module %s does not have a function with the same name!" % (
                funname))
        else:
            # insert into THIS namespace
            g = globals()
            g[funname] = mod.__dict__[funname]
            names.append(funname)
    
    return names


# do it and clean up
_functionNames = _insertFunctions()



#!/usr/bin/env python

""" This example shows how to use the same camera for multiple axes,
which can be helpful if for example the axes show a different view
on the same data.
"""

import visvis as vv
app = vv.use()

# Read lena
im1 = vv.imread('astronaut.png')

# Our second image is a thresholded image
im2 = im1 > 100

# Create figure with two axes
vv.figure()
a1 = vv.subplot(121)
a2 = vv.subplot(122)

# Create new camera and attach
cam = vv.cameras.TwoDCamera()
a1.camera = a2.camera = cam

# Draw images
vv.imshow(im1, axes=a1)
vv.imshow(im2, axes=a2)

app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv


class Bars3D(vv.Wobject):
    """ Bars3D(parent)
    
    The Bars3D class represents a bar chart. It has a few methods to
    change the appearance of the bars.
    
    This class is a container for a series of Mesh objects created using
    solidBox.
    
    This wobject is created by the function vv.bar3().
    
    """
    
    @vv.misc.Property
    def ambient():
        """ Get/Set the ambient reflectance of the bars.
        The getter returns the ambient reflectance of the first bar,
        or None if there are no bars.
        """
        def fget(self):
            if self._children:
                return self._children[0].ambient
            else:
                return None
        def fset(self, value):
            for child in self.children:
                child.ambient = value
        return locals()
    
    @vv.misc.Property
    def diffuse():
        """ Get/Set the diffuse reflectance of the bars.
        The getter returns the diffuse reflectance of the first bar,
        or None if there are no bars.
        """
        def fget(self):
            if self._children:
                return self._children[0].diffuse
            else:
                return None
        def fset(self, value):
            for child in self.children:
                child.diffuse = value
        return locals()
    
    @vv.misc.Property
    def specular():
        """ Get/Set the specular reflectance of the bars.
        The getter returns the specular reflectance of the first bar,
        or None if there are no bars.
        """
        def fget(self):
            if self._children:
                return self._children[0].specular
            else:
                return None
        def fset(self, value):
            for child in self.children:
                child.specular = value
        return locals()
    
    @vv.misc.Property
    def color():
        """ Get/Set the color of the bars. The getter returns the color
        of the first bar, or None if there are no bars.
        """
        def fget(self):
            if self._children:
                return self._children[0].faceColor
            else:
                return None
        def fset(self, value):
            for child in self.children:
                child.faceColor = value
        return locals()
    
    @vv.misc.Property
    def colors():
        """ Get/Set the colors of all bars at once. The number of colors
        should match the number of bars. """
        def fget(self):
            if self._children:
                return [child.faceColor for child in self.children
                                    if hasattr(child, 'faceColor')]
            else:
                return None
        def fset(self, value):
            for child, color in zip(self.children, value):
                child.faceColor = color
        return locals()



def bar3(data1, data2=None, data3=None, width=0.75, axesAdjust=True, axes=None):
    """ bar3(*args, width=0.75, axesAdjust=True, axes=None)
    
    Create a 3D bar chart and returns a Bars3D instance that can be
    used to change the appearance of the bars (such as lighting properties
    and color).
    
    Usage
    -----
      * bar3(H, ...) creates bars of specified height.
      * bar3(X, H, ...) also supply their x-coordinates
      * bar3(X, Y, H, ...) supply both x- and y-coordinates
    
    Keyword arguments
    -----------------
    width : scalar
        The width of the bars.
    axesAdjust : bool
        If True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet,
        it is set to False.
    axes : Axes instance
        Display the bars in the given axes, or the current axes if not given.
    
    """
    
    # Pre check
    if True:
        try:
            if not hasattr(data1, '__len__'):
                raise ValueError
            data1 = [float(val) for val in data1]
        except Exception:
            raise ValueError('bar3 needs a sequence of numbers.')
    if data2 is not None:
        try:
            if not hasattr(data2, '__len__'):
                raise ValueError
            data2 = [float(val) for val in data2]
        except Exception:
            raise ValueError('bar3 needs a sequence of numbers.')
    if data3 is not None:
        try:
            if not hasattr(data3, '__len__'):
                raise ValueError
            data3 = [float(val) for val in data3]
        except Exception:
            raise ValueError('bar3 needs a sequence of numbers.')
    
    # Parse input
    if data2 is None and data3 is None:
        # Only height given
        hh = data1
        xx = list(range(len(hh)))
        yy = [0] * len(hh)
    elif data3 is None:
        # Height and x given
        xx = data1
        hh = data2
        yy = [0] * len(hh)
    else:
        # All three given
        xx = data1
        yy = data2
        hh = data3
    
    # Check
    if len(hh) != len(xx) or len(hh) != len(yy):
        raise ValueError('Given arrays for bar3 must be of equal length.')
    
    # Get axes
    if axes is None:
        axes = vv.gca()
    
    # Create Bars instance
    bars = Bars3D(axes)
    
    # Create boxes
    for x,y,h in zip(xx,yy, hh):
        pos = (x,y,h/2.0)
        scale = (width,width,h)
        m = vv.solidBox(pos, scale, axesAdjust=False, axes=bars)
        m.specular = 0
    
   # Adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = False
        axes.cameraType = '3d'
        axes.SetLimits()
    
    # Done
    axes.Draw()
    return bars
    
    
    
if __name__ == '__main__':
    a = vv.gca()
    b = vv.bar3([1,2,3,2,4,3], width=0.5)
    a.axis.showGrid = 1
    a.axis.xTicks = ['Januari', 'Februari', 'March', 'April', 'May', 'June']
    b.colors = 'rgbcmy'



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import os
import visvis as vv

# Try importing imageio
imageio = None

try:
    import imageio
except ImportError:
    pass


def movieRead(filename, *args, **kwargs):
    """ Proxy for imageio.mimread()
    """
    
    if imageio is None:
        raise RuntimeError("visvis.movieRead requires the imageio package.")
    
    if not os.path.isfile(filename):
        # try loadingpil from the resource dir
        path = vv.misc.getResourceDir()
        filename2 = os.path.join(path, filename)
        if os.path.isfile(filename2):
            filename = filename2
    
    return imageio.mimread(filename, *args, **kwargs)


if __name__ == '__main__':
    ims = vv.movieRead('newtonscradle.gif')
    vv.movieShow(ims)



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module baseWobjects

Defines the mix class for orientable wobjects.

"""

import numpy as np
from visvis.core import misc
from visvis.utils.pypoints import Point, is_Point


# todo: is this the best way to allow users to orient their objects,
# or might there be other ways?
class OrientationForWobjects_mixClass(object):
    """ OrientationForWobjects_mixClass()
    
    This class can be mixed with a wobject class to enable easy
    orientation of the objects in space. It makes use of the
    tranformation list that each wobject has.
    
    The functionality provided by this class is not made part of the
    Wobject class because it does not make sense for all kind of wobjects
    (for example lines and images). The OrientableMesh is a class that
    inherits from this class.
    
    """
    
    def __init__(self):
        
        # Set current and reference direction (default up)
        self._refDirection = Point(0,0,1)
        self._direction = Point(0,0,1)
        
        # Create transformations
        self._scaleTransform = misc.Transform_Scale()
        self._translateTransform = misc.Transform_Translate()
        self._rotateTransform = misc.Transform_Rotate()
        self._directionTransform = misc.Transform_Rotate()
        
        # Append transformations to THE list
        self.transformations.append(self._translateTransform)
        self.transformations.append(self._directionTransform)
        self.transformations.append(self._rotateTransform)
        self.transformations.append(self._scaleTransform)
    
    
    @misc.PropWithDraw
    def scaling():
        """ Get/Set the scaling of the object. Can be set using
        a 3-element tuple, a 3D point, or a scalar. The getter always
        returns a Point.
        """
        def fget(self):
            s = self._scaleTransform
            return Point(s.sx, s.sy, s.sz)
        def fset(self, value):
            if isinstance(value, (float, int)):
                self._scaleTransform.sx = float(value)
                self._scaleTransform.sy = float(value)
                self._scaleTransform.sz = float(value)
            elif isinstance(value, (list, tuple)) and len(value) == 3:
                self._scaleTransform.sx = float(value[0])
                self._scaleTransform.sy = float(value[1])
                self._scaleTransform.sz = float(value[2])
            elif is_Point(value) and value.ndim == 3:
                self._scaleTransform.sx = value.x
                self._scaleTransform.sy = value.y
                self._scaleTransform.sz = value.z
            else:
                raise ValueError('Scaling should be a scalar, 3D Point, or 3-element tuple.')
        return locals()
    
    
    @misc.PropWithDraw
    def translation():
        """ Get/Set the transaltion of the object. Can be set using
        a 3-element tuple or a 3D point. The getter always returns
        a Point.
        """
        def fget(self):
            d = self._translateTransform
            return Point(d.dx, d.dy, d.dz)
        def fset(self, value):
            if isinstance(value, (list, tuple)) and len(value) == 3:
                self._translateTransform.dx = value[0]
                self._translateTransform.dy = value[1]
                self._translateTransform.dz = value[2]
            elif is_Point(value) and value.ndim == 3:
                self._translateTransform.dx = value.x
                self._translateTransform.dy = value.y
                self._translateTransform.dz = value.z
            else:
                raise ValueError('Translation should be a 3D Point or 3-element tuple.')
        return locals()
    
    
    @misc.PropWithDraw
    def direction():
        """ Get/Set the direction (i.e. orientation) of the object. Can
        be set using a 3-element tuple or a 3D point. The getter always
        returns a Point.
        """
        def fget(self):
            return self._direction.copy()
        def fset(self, value):
            # Store direction
            if isinstance(value, (list, tuple)) and len(value) == 3:
                self._direction = Point(*tuple(value))
            elif is_Point(value) and value.ndim == 3:
                self._direction = value
            else:
                raise ValueError('Direction should be a 3D Point or 3-element tuple.')
            
            # Normalize
            if self._direction.norm()==0:
                raise ValueError('Direction vector must have a non-zero length.')
            self._direction = self._direction.normalize()
            
            # Create ref point
            refPoint = self._refDirection
            
            # Convert to rotation. The cross product of two vectors results
            # in a vector normal to both vectors. This is the axis of rotation
            # over which the minimal rotation is achieved.
            axis = self._direction.cross(refPoint)
            if axis.norm() < 0.01:
                if self._direction.z > 0:
                    # No rotation
                    self._directionTransform.ax = 0.0
                    self._directionTransform.ay = 0.0
                    self._directionTransform.az = 1.0
                    self._directionTransform.angle = 0.0
                else:
                    # Flipped
                    self._directionTransform.ax = 1.0
                    self._directionTransform.ay = 0.0
                    self._directionTransform.az = 0.0
                    self._directionTransform.angle = 180.0
            else:
                axis = axis.normalize()
                angle = -refPoint.angle(self._direction)
                self._directionTransform.ax = axis.x
                self._directionTransform.ay = axis.y
                self._directionTransform.az = axis.z
                self._directionTransform.angle = angle * 180 / np.pi
        return locals()
    
    
    @misc.PropWithDraw
    def rotation():
        """ Get/Set the rotation of the object (in degrees, around its
        direction vector).
        """
        def fget(self):
            return self._rotateTransform.angle
        def fset(self, value):
            self._rotateTransform.angle = float(value)
        return locals()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Package visvis.utils

This package contains a few modules and packages that are stand-alone
and maybe useful outside visvis. Some utilities are used inside visvis
(such as ssdf), others extend the functionality of visvis or have some
relation to visualization.

The intended way to import utilities is: "from visvis.utils import utilname".

The currrent utilities are:

  * ssdf - simple structured data format.
  * pypoints - Representnig points, pointsets, anisotropic arrays and quaternions.
  * graph - Representing points/nodes that are connected by edges.
  * cropper - small app based on visvis controls to crop 3D data

"""



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv

def grid(*args, **kwargs):
    """ grid(*args, axesAdjust=True, axes=None)
    
    Create a wireframe parametric surface.
    
    Usage
    -----
      * grid(Z) - create a grid mesh using the given image with z coordinates.
      * grid(Z, C) - also supply a texture image to map.
      * grid(X, Y, Z) - give x, y and z coordinates.
      * grid(X, Y, Z, C) - also supply a texture image to map.
    
    Parameters
    ----------
    Z : A MxN 2D array
    X : A length N 1D array, or a MxN 2D array
    Y : A length M 1D array, or a MxN 2D array
    C : A MxN 2D array, or a AxBx3 3D array
        If 2D, C specifies a colormap index for each vertex of Z.  If
        3D, C gives a RGB image to be mapped over Z.  In this case, the
        sizes of C and Z need not match.
    
    Keyword arguments
    -----------------
    axesAdjust : bool
        If True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet,
        it is set to False.
    axes : Axes instance
        Display the bars in the given axes, or the current axes if not given.
    
    Notes
    -----
      * This function should not be confused with the axis grid, see the
        Axis.showGrid property.
      * This function is know in Matlab as mesh(), but to avoid confusion
        with the vv.Mesh class, it is called grid() in visvis.
    
    Also see surf() and the solid*() methods.
    
    """
    
    m = vv.surf(*args, **kwargs)
    m.faceShading = None
    m.edgeShading = 'smooth'
    m.edgeColor = 'w'
    return m

if __name__ == '__main__':
    vv.figure()
    m = vv.grid(vv.peaks())
    m.colormap = vv.CM_HOT



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np

def volshow2(vol, clim=None, renderStyle=None, cm=None,
            axesAdjust=True, axes=None):
    """ volshow2(vol, clim=None, renderStyle=None, cm=CM_GRAY,
            axesAdjust=True, axes=None)
    
    Display a 3D image (a volume) using three slice textures that can be
    moved interactively. Returns a SliceTextureProxy instance that "wraps"
    the three SliceTexture instances.
    
    Parameters
    ----------
    vol : numpy array
        The 3D image to visualize. Can be grayscale, RGB, or RGBA.
        If the volume is an anisotropic array (vv.Aaray), the appropriate
        scale and translate transformations are applied.
    clim : 2-element tuple
        The color limits to scale the intensities of the image. If not given,
        the im.min() and im.max() are used (neglecting nan and inf).
    renderStyle : not used
        This parameter is included for compatibility with volshow3. Its
        value is ignored.
    cm : Colormap
        Set the colormap to apply in case the volume is grayscale.
    axesAdjust : bool
        If axesAdjust==True, this function will call axes.SetLimits(), and set
        the camera type to 3D. If daspectAuto has not been set yet, it is
        set to False.
    axes : Axes instance
        Display the image in this axes, or the current axes if not given.
    
    """
    
    # get axes
    if axes is None:
        axes = vv.gca()
    
     # Check data
    if not isinstance(vol, np.ndarray):
        raise ValueError('volshow expects an image as a numpy array.')
    elif vol.size==0:
        raise ValueError('volshow cannot draw arrays with zero elements.')
    #
    if vol.ndim==3 or vol.ndim==4 and vol.shape[-1] in [1,3,4]:
        pass
    else:
        raise ValueError('volshow expects a 3D image as a numpy array.')
    
    # Get centre
    centre = [int(s/2) for s in vol.shape]
    
    # create textures
    proxy = vv.SliceTextureProxy(axes)
    for i in range(3):
        vv.SliceTexture(proxy, vol, i, centre[i])
    
    # set clim
    if isinstance(clim,list):
        clim = tuple(clim)
    if isinstance(clim, tuple):
        proxy.SetClim(clim)
    
    # set colormap
    if cm is not None:
        proxy.colormap = cm
    
    # adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = False
        axes.cameraType = '3d'
        axes.SetLimits()
    
    # done
    axes.Draw()
    return proxy


if __name__ == "__main__":
    vol = vv.aVolume()
    t = vv.volshow2(vol)



#!/usr/bin/env python
""" This example illustrates how to draw lines in an axis,
for instance for creating annotations.
"""

import visvis as vv
from visvis import Point, Pointset

class Drawer:
    
    def __init__(self):
        
        # Create figure and axes
        vv.figure()
        self._a = a = vv.gca()
        vv.title("Hold mouse to draw lines. Use 'rgbcmyk' and '1-9' keys.")
        
        # Set axes
        a.SetLimits((0,1), (0,1))
        a.cameraType = '2d'
        a.daspectAuto = False
        a.axis.showGrid = True
        
        # Init variables needed during drawing
        self._active = None
        self._pp = Pointset(2)
        
        # Create null and empty line objects
        self._line1 = None
        self._line2 = vv.plot(vv.Pointset(2), ls='+', lc='c', lw='2', axes=a)
        
        # Bind to events
        a.eventMouseDown.Bind(self.OnDown)
        a.eventMouseUp.Bind(self.OnUp)
        a.eventMotion.Bind(self.OnMotion)
        a.eventKeyDown.Bind(self.OnKey)
    
    
    def OnKey(self, event):
        """ Called when a key is pressed down in the axes.
        """
        
        if event.text and event.text.lower() in 'rgbcmywk':
            self._line2.lc = event.text.lower()
        else:
            try:
                tickness = int(event.text)
            except Exception:
                tickness = 0
            if tickness > 0:
                self._line2.lw = tickness
    
    
    def OnDown(self, event):
        """ Called when the mouse is pressed down in the axes.
        """
        
        # Only process left mouse button
        if event.button != 1:
            return False
        
        # Store location
        self._active = Point(event.x2d, event.y2d)
        
        # Clear temp line object
        if self._line1:
            self._line1.Destroy()
        
        # Create line objects
        tmp = Pointset(2)
        tmp.append(self._active)
        tmp.append(self._active)
        self._line1 = vv.plot(tmp, lc='r', lw='1', axes=self._a, axesAdjust=0)
        
        # Draw
        self._a.Draw()
        
        # Prevent dragging by indicating the event needs no further handling
        return True

    
    def OnMotion(self, event):
        """ Called when the mouse is moved in the axes.
        """
        
        if self._active and self._line1:
            # Update line
            tmp = Pointset(2)
            tmp.append(self._active)
            tmp.append(event.x2d, event.y2d)
            self._line1.SetPoints(tmp)
            # Draw
            self._a.Draw()
    
    
    def OnUp(self, event):
        """ Called when the mouse is released (when first pressed down
        in the axes).
        """
        
        # Only if a point is active
        if self._active is None:
            return False
        
        # Get points
        p1 = self._active
        p2 = Point(event.x2d, event.y2d)
        
        # Add!
        self._pp.append(p1)
        self._pp.append(p2)
        
        # We're done with this one
        self._active = None
        
        # Clear temp line object
        if self._line1:
            self._line1.Destroy()
            self._line1 = None
        
        # Update lines
        self._line2.SetPoints(self._pp)
        
        # Draw
        self._a.Draw()


if __name__ == '__main__':
    d = Drawer()
    app = vv.use()
    app.Run()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

""" Module light

Defines a light source to light polygonal surfaces. Each axes has up
to eight lights associated with it.

"""

import OpenGL.GL as gl

from visvis.core import misc
from visvis.core.misc import PropWithDraw, DrawAfter, basestring


def _testColor(value, canBeScalar=True):
    """ _testColor(value, canBeScalar=True)
    
    Tests a color whether it is a sequence of 3 or 4 values.
    It returns a 4 element tuple or raises an error if the suplied
    data is incorrect.
    
    """
    
    # Deal with named colors
    if isinstance(value, basestring):
        value = misc.getColor(value)
    
    # Value can be a scalar
    if canBeScalar and isinstance(value, (int, float)):
        if value <= 0:
            value = 0.0
        if value >= 1:
            value = 1.0
        return value
    
    # Otherwise it must be a sequence of 3 or 4 elements
    elif not hasattr(value, '__len__'):
        raise ValueError("Given value can not represent a color.")
    elif len(value) == 4:
        return (value[0], value[1], value[2], value[3])
    elif len(value) == 3:
        return (value[0], value[1], value[2], 1.0)
    else:
        raise ValueError("Given value can not represent a color.")


def _getColor(color, ref):
    """ _getColor(color, reference)
    
    Get the real color as a 4 element tuple, using the reference
    color if the given color is a scalar.
    
    """
    if isinstance(color, float):
        return (color*ref[0], color*ref[1], color*ref[2], ref[3])
    else:
        return color



# todo: implement spot light and attenuation
class Light(object):
    """ Light(axes, index)
    
    A Light object represents a light source in the scene. It
    determines how lit objects (such as Mesh objects) are visualized.
    
    Each axes has 8 light sources, of which the 0th is turned on
    by default. De 0th light source provides the ambient light in the
    scene (the ambient component is 0 by default for the other light
    sources). Obtain the lights using the axes.light0 and axes.lights
    properties.
    
    The 0th light source is a directional camera light by default; it
    shines in the direction in which you look. The other lights are
    oriented at the origin by default.
    
    """
    
    def __init__(self, axes, index):
        
        # Store axes and index of the light (OpenGl can handle up to 8 lights)
        self._axes = axes.GetWeakref()
        self._index = index
        self._on = False
        
        # The three light properties
        self._color = (1, 1, 1, 1)
        self._ambient = 0.0
        self._diffuse = 1.0
        self._specular = 1.0
        
        # The main light has an ambien component by default
        if index == 0:
            self._ambient = 0.2
        
        # Position or direction
        if index == 0:
            self._position = (0,0,1,0)
            self._camLight = True
        else:
            self._position = (0,0,0,1)
            self._camLight = False
    
    
    def Draw(self):
        # Draw axes
        axes = self._axes()
        if axes:
            axes.Draw()
    
    @PropWithDraw
    def color():
        """ Get/Set the reference color of the light. If the ambient,
        diffuse or specular properties specify a scalar, that scalar
        represents the fraction of *this* color.
        """
        def fget(self):
            return self._color
        def fset(self, value):
            self._color = _testColor(value, True)
        return locals()
    
    
    @PropWithDraw
    def ambient():
        """ Get/Set the ambient color of the light. This is the color
        that is everywhere, coming from all directions, independent of
        the light position.
        
        The value can be a 3- or 4-element tuple, a character in
        "rgbycmkw", or a scalar between 0 and 1 that indicates the
        fraction of the reference color.
        """
        def fget(self):
            return self._ambient
        def fset(self, value):
            self._ambient = _testColor(value)
        return locals()
    
    
    @PropWithDraw
    def diffuse():
        """ Get/Set the diffuse color of the light. This component is the
        light that comes from one direction, so it's brighter if it comes
        squarely down on a surface than if it barely glances off the
        surface. It depends on the light position how a material is lit.
        """
        def fget(self):
            return self._diffuse
        def fset(self, value):
            self._diffuse = _testColor(value)
        return locals()
    
    
    @PropWithDraw
    def specular():
        """ Get/Set the specular color of the light. This component
        represents the light that comes from the light source and bounces
        off a surface in a particular direction. This is what makes
        materials appear shiny.
        
        The value can be a 3- or 4-element tuple, a character in
        "rgbycmkw", or a scalar between 0 and 1 that indicates the
        fraction of the reference color.
        """
        def fget(self):
            return self._specular
        def fset(self, value):
            self._specular = _testColor(value)
        return locals()
    
    
    @PropWithDraw
    def position():
        """ Get/Set the position of the light. Can be represented as a
        3 or 4 element tuple. If the fourth element is a 1, the light
        has a position, if it is a 0, it represents a direction (i.o.w. the
        light is a directional light, like the sun).
        """
        def fget(self):
            return self._position
        def fset(self, value):
            if len(value) == 3:
                self._position = value[0], value[1], value[2], 1
            elif len(value) == 4:
                self._position = value[0], value[1], value[2], value[3]
            else:
                tmp = "Light position should be a 3 or 4 element sequence."
                raise ValueError(tmp)
        return locals()
    
    
    @PropWithDraw
    def isDirectional():
        """ Get/Set whether the light is a directional light. A directional
        light has no real position (it can be thought of as infinitely far
        away), but shines in a particular direction. The sun is a good
        example of a directional light.
        """
        def fget(self):
            return self._position[3] == 0
        def fset(self, value):
            # Get fourth element
            if value:
                fourth = 0
            else:
                fourth = 1
            # Set position
            tmp = self._position
            self._position = tmp[0], tmp[1], tmp[2], fourth
        return locals()
    
    
    @PropWithDraw
    def isCamLight():
        """ Get/Set whether the light is a camera light. A camera light
        moves along with the camera, like the lamp on a miner's hat.
        """
        def fget(self):
            return self._camLight
        def fset(self, value):
            self._camLight = bool(value)
        return locals()
    
    
    @DrawAfter
    def On(self, on=True):
        """ On(on=True)
        Turn the light on.
        """
        self._on = bool(on)
    
    
    @DrawAfter
    def Off(self):
        """ Off()
        Turn the light off.
        """
        self._on = False
    
    
    @property
    def isOn(self):
        """ Get whether the light is on.
        """
        return self._on
    
    
    def _Apply(self):
        """ _Apply()
        
        Apply the light position and other properties.
        
        """
        thisLight = gl.GL_LIGHT0 + self._index
        if self._on:
            # Enable and set position
            gl.glEnable(thisLight)
            gl.glLightfv(thisLight, gl.GL_POSITION, self._position)
            # Set colors
            amb, dif, spe = gl.GL_AMBIENT, gl.GL_DIFFUSE, gl.GL_SPECULAR
            gl.glLightfv(thisLight, amb, _getColor(self._ambient, self._color))
            gl.glLightfv(thisLight, dif, _getColor(self._diffuse, self._color))
            gl.glLightfv(thisLight, spe, _getColor(self._specular, self._color))
        else:
            # null-position means that the ligth is off
            gl.glLightfv(thisLight, gl.GL_POSITION, (0.0, 0.0, 0.0, 0.0))
            gl.glDisable(thisLight)



# -*- coding: utf-8 -*-
# flake8: noqa
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.


""" Subpackage visvis.text

For rendering text in visvis.

Defines a wibject and a wobject: Label and Text,
which are both able to produce a single line of text
oriented at a certain angle.

Visvis used the FreeType library to render text, enabling good-looking text
(with proper kerning and anti-aliasing) at any size. If FreeType is not
available, visvis falls back on system that uses pre-rendered fonts.
FreeType is available:

  * On Linux: by default
  * On Windows: needs to be installed (there is an installer on the visvis website)
  * On Mac: probably needs to be installed (also fc-match if you want to
    make use of other fonts then the default.


Formatting
----------

Text can be formatted using the following constructs (which can be mixed):
  * hello^2 or hello^{there}, makes one or more charactes superscript.
  * hello_2 or hello_{there}, makes one or more charactes subscript.
  * hell\io or hell\i{ohoo}, makes one or more charactes italic.
  * hell\bo or hell\b{ohoo}, makes one or more charactes bold.
  * hello\_there,  a backslash escapes, thus keeping the _^ or \ after it.


Special characters
------------------

Characters are available for the following unicode sets:
  * u0020 - u003f  numbers
  * u0040 - u00bf  alphabet
  * u00c0 - u037f  latin
  * u0380 - u03ff  greek
  * u2000 - u23ff  symbols

There are several escape sequences for (mathematical) characters
that can be inserted using the backslash (for example '\infty').
People familiar with Latex know what they do:
  * Re          Im          null        infty
  * int         iint        iiint       forall
  * leq         geq         approx      approxeq        ne          in
  * leftarrow   uparrow     rightarrow  downarrow
  * Leftarrow   Uparrow     Rightarrow  Downarrow
  * leftceil    rightceil   leftfloor   rightfloor
  * times       cdot        pm
  * oplus       ominus      otimes      oslash

Letters from the greek alfabet can be inserted in the same
way (By starting the name with an uppercase letter, the
corresponding upper case greek letter is inserted):
  * alpha       beta        gamma       delta
  * epsilon     zeta        eta         theta
  * iota        kappa       lambda      mu
  * nu          xi          omicron     pi
  * rho         varsigma    sigma       tau
  * upsilon     phi         chi         psi
  * omega

Note: In case one needs a character that is not in this list,
one can always look up its unicode value and use that instead.

"""

from visvis.text.text_base import AtlasTexture, FontManager, BaseText
from visvis.text.text_base import Text, Label

from visvis.text.text_prerendered import PrerenderedFontManager

# Try importing the freetype font manager
# Note that FreeType must be installed. On Linux this would usually be
# the case, otherwise you can install it via the package manager.
# For Windows users there is a simple installer on the Visvis website.
# For Mac I'm not sure yet.
try:
    from visvis.text.text_freetype import FreeTypeFontManager
except RuntimeError:
    FreeTypeFontManager = None

if FreeTypeFontManager is not None:
    FontManager = FreeTypeFontManager
else:
    FontManager = PrerenderedFontManager



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Robert Schroll
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
from visvis.wobjects.polygonalModeling import checkDimsOfArray


def mesh(vertices, faces=None, normals=None, values=None, verticesPerFace=3,
        colormap=None, clim=None, texture=None, axesAdjust=True, axes=None):
    """ mesh(vertices, faces=None, normals=None, values=None, verticesPerFace=3,
        colormap=None, clim=None, texture=None, axesAdjust=True, axes=None)
    
    Display a mesh of polygons, either triangles or quads.
    
    Parameters
    ----------
    vertices : Nx3 array
        The positions of the vertices in 3D space.
    faces : array or list of indices
        The faces given in terms of the vertex indices.  Should be 1D, in
        which case the indices are grouped into groups of verticesPerFace,
        or Mx3 or Mx4, in which case verticesPerFace is ignored.
        The front of the face is defined using the right-hand-rule.
    normals : Nx3
        A list of vectors specifying the vertex normals.
    values : N, Nx2, Nx3, or Nx4 array
        Sets the color of each vertex, using values from a colormap (1D),
        colors from a texture (Nx2), RGB values (Nx3), or RGBA value (Nx4).
    verticesPerFace : 3 or 4
        Whether the faces are triangle or quads, if not specified in faces.
    colormap : a Colormap
        If values is 1D, the vertex colors are set from this colormap.
    clim : 2 element array
        If values is 1D, sets the values to be mapped to the limits of the
        colormap.  If None, the min and max of values are used.
    texture : a Texture
        If values is Nx2, the vertex colors are set from this texture.
    axesAdjust : bool
        Whether to adjust the view after the mesh is drawn.
    axes : Axes instance
        The axes into which the mesh will be added.  If None, the current
        axes will be used.
    
    """
    
    if axes is None:
        axes = vv.gca()
    
    # Accept basemesh instances
    if isinstance(vertices, vv.BaseMesh):
        other = vertices
        vertices = other._vertices
        faces = other._faces
        normals = other._normals
        values = other._values
        verticesPerFace = other._verticesPerFace
    
    # Check that vertices is (converted to) a Nx3 array; otherwise user
    # will see odd behavior from Mesh
    if not isinstance(vertices, vv.BaseMesh):
        try:
            vertices = checkDimsOfArray(vertices, 3)
        except ValueError:
            raise ValueError("Vertices should represent an array of 3D vertices.")
    
    # Set the method for coloring
    if values is not None:
        try:
            values = checkDimsOfArray(values, 0, 1, 2, 3, 4)
            # Returned values is always a 2D numpy array
        except ValueError:
            raise ValueError('values must be 1D, Nx2, Nx3, or Nx4 array.')
        if values.shape[0] != vertices.shape[0]:
            raise ValueError('First dimension of values must be same length as vertices.')
    
    # Instantiate Mesh
    m = vv.Mesh(axes, vertices, faces, normals, values, verticesPerFace)
    
    # Set colormap or texture
    if values is not None and values.shape[1] == 1:
        if colormap is not None:
            m.colormap = colormap
        if clim is not None and len(clim) == 2:
            m.clim = clim
        else:
            m.clim = values.min(), values.max()
    elif texture is not None and values is not None and values.shape[1] == 2:
        m.SetTexture(texture)
    
    # Adjust axes
    if axesAdjust:
        if axes.daspectAuto is None:
            axes.daspectAuto = False
        axes.cameraType = '3d'
        axes.SetLimits()
    
    # Return
    axes.Draw()
    return m


if __name__ == '__main__':
    # Create BaseMesh object (has no visualization props)
    bm = vv.meshRead('teapot.ssdf')
    # Show it, returning a Mesh object (which does have visualization props)
    m = vv.mesh(bm)



#!/usr/bin/env python
"""
This example illustrates embedding a visvis figure in a Qt application.
This example works for the pyqt4 and pyside backends.
"""

try:
    from PySide import QtGui, QtCore  # noqa
    backend = 'pyside'
except ImportError:
    from PyQt4 import QtGui, QtCore  # noqa
    backend = 'pyqt4'

import visvis as vv

# Create a visvis app instance, which wraps a qt4 application object.
# This needs to be done *before* instantiating the main window.
app = vv.use(backend)

class MainWindow(QtGui.QWidget):
    def __init__(self, *args):
        QtGui.QWidget.__init__(self, *args)
        
        # Make a panel with a button
        self.panel = QtGui.QWidget(self)
        but = QtGui.QPushButton(self.panel)
        but.setText('Push me')
        
        # Make figure using "self" as a parent
        Figure = app.GetFigureClass()
        self.fig = Figure(self)
        
        # Make sizer and embed stuff
        self.sizer = QtGui.QHBoxLayout(self)
        self.sizer.addWidget(self.panel, 1)
        self.sizer.addWidget(self.fig._widget, 2)
        
        # Make callback
        but.pressed.connect(self._Plot)
        
        # Apply sizers
        self.setLayout(self.sizer)
        
        # Finish
        self.resize(560, 420)
        self.setWindowTitle('Embedding in Qt (%s)' % backend)
        self.show()
    
    
    def _Plot(self):
        
        # Make sure our figure is the active one.
        # If only one figure, this is not necessary.
        #vv.figure(self.fig.nr)
        
        # Clear it
        vv.clf()
        
        # Plot
        vv.plot([1,2,3,1,6])
        vv.legend(['this is a line'])
        #self.fig.DrawNow()
    

# Two ways to create the application and start the main loop
if True:
    # The visvis way. Will run in interactive mode when used in IEP or IPython.
    app.Create()
    m = MainWindow()
    app.Run()

else:
    # The native way.
    qtApp = QtGui.QApplication([''])
    m = MainWindow()
    qtApp.exec_()



# -*- coding: utf-8 -*-
# Copyright (C) 2012, Almar Klein
#
# Visvis is distributed under the terms of the (new) BSD License.
# The full license can be found in 'license.txt'.

import visvis as vv
import numpy as np

def kde(data, bins=None, kernel=None, **kwargs):
    """ kde(a, bins=None, range=None, **kwargs)
    
    Make a kernerl density estimate plot of the data. This is like a
    histogram, but produces a smoother result, thereby better represening
    the probability density function.
    
    See the vv.StatData for more statistics on data.
    
    Parameters
    ----------
    a : array_like
        The data to calculate the historgam of.
    bins : int (optional)
        The number of bins. If not given, the best number of bins is
        determined automatically using the Freedman-Diaconis rule.
    kernel : float or sequence (optional)
        The kernel to use for distributing the values. If a scalar is given,
        a Gaussian kernel with a sigma equal to the given number is used.
        If not given, the best kernel is chosen baded on the number of bins.
    kwargs : keyword arguments
        These are given to the plot function.
    
    """
    
    # Get stats
    from visvis.processing.statistics import StatData
    stats = StatData(data)
    
    # Get kde
    xx, values = stats.kde(bins, kernel)
    
    # Plot
    return vv.plot(xx, values, **kwargs)


if __name__ == '__main__':
    vv.clf()
    data = np.random.normal(7,2,size=(100,100))
    b = vv.kde(data, lc='r')
